@@ -1,23 +1,34 @@
-# $Id: crt.py 1470 2016-03-28 18:36:17Z mwall $
+# $Id: crt.py 1477 2016-04-10 13:19:56Z mwall $
 # Copyright 2013-2016 Matthew Wall
 # thanks to gary roderick for significant contributions to this code
 
 """Emit loop data to file in Cumulus realtime format.
+
    http://wiki.sandaysoft.com/a/Realtime.txt
+   http://wiki.sandaysoft.com/a/Webtags
 
 Put this file in bin/user/crt.py, then add this to your weewx.conf:
 
 [CumulusRealTime]
-    unit_system = METRIC # options are US, METRIC, METRICWX
-    date_separator = /
     filename = /path/to/realtime.txt
-    none = NULL
 
 [Engine]
     [[Services]]
         archive_services = ..., user.crt.CumulusRealTime
 
 If no unit_system is specified, the units will be those of the database.
+Units and other parameters may be specified:
+
+[CumulusRealTime]
+    filename = /path/to/realtime.txt
+    date_separator = /
+    none = NULL
+    unit_system = (US | METRIC | METRICWX)
+    wind_units = (meter_per_second | mile_per_hour | km_per_hour | knot)
+    temperature_units = (degree_C | degree_F)
+    pressure_units = (hPa | mbar | inHg)
+    rain_units = (mm | inch)
+    cloudbase_units = (foot | meter)
 
 Note that most of the code in this file is to calculate/lookup data that
 are not directly provided by weewx in a LOOP packet.
@@ -27,19 +38,19 @@ The cumulus 'specification' for realtime.txt is ambiguous in places:
   - pressure trend interval is not specified, we use 3 hours for pressure
   - temperature trend interval is not specified, we use 3 hours for temperature
   
-The following assumptions have been made based on the equivalent Cumulus 
-Webtags listed at http://wiki.sandaysoft.com/a/Realtime.txt and 
-  http://wiki.sandaysoft.com/a/Webtags:
+The following assumptions are based on the Cumulus realtime and webtag docs:
   - wind avg speed/wind avg dir interval is not specified. The equivalent 
     Cumulus Webtags for wind avg speed and avg wind direction default to 
     10 minute averages so we use 10 minutes as well.
   - wind direction bearings in degrees are set from > 0 to 360 with 0
-    indicating calm (refer to the #avgbearing Webtag at
-       http://wiki.sandaysoft.com/a/Webtags) 
-    (this is different to the standard used in weewx)
+    indicating calm.  refer to the #avgbearing Webtag. this is different to
+    the standard used in weewx.
+  - possible Cumulus windrun units are km, miles, km and nm corresponding to 
+    wind speeds in m/s, mph, km/h, kts.  weewx does not know what a nautical
+    mile (nm) is.
   
-The following assumptions have been made based on examination of realtime.txt
-instances from a number of live Cumulus sites:
+The following assumptions are based on examination of realtime.txt instances
+from a number of live Cumulus sites:
   - time zone is not specified, local time is used throughout
   - how to handle None/NULL is not specified.  Examination of realtime.txt 
     from a number of live Cumulus sites indicates when there is no wind 
@@ -49,6 +60,9 @@ instances from a number of live Cumulus sites:
   - ordinal wind directions are set to --- when there is no wind.
 """
 
+# FIXME: consider in-memory caching so that database queries are not
+#        necessary after the first invocation
+
 import math
 import time
 import syslog
@@ -62,9 +76,9 @@ import weeutil.weeutil
 import weedb
 from weewx.engine import StdService
 
-VERSION = "0.17"
+VERSION = "0.18"
 
-REQUIRED_WEEWX = "3.2.0"
+REQUIRED_WEEWX = "3.5.0"
 if StrictVersion(weewx.__version__) < StrictVersion(REQUIRED_WEEWX):
     raise weewx.UnsupportedFeature("weewx %s or greater is required, found %s"
                                    % (REQUIRED_WEEWX, weewx.__version__))
@@ -85,26 +99,42 @@ COMPASS_POINTS = ['N', 'NNE', 'NE', 'ENE', 'E', 'ESE', 'SE', 'SSE',
                   'S', 'SSW', 'SW', 'WSW', 'W', 'WNW', 'NW', 'NNW', 'N']
 
 # map weewx unit names to cumulus unit names
+UNITS_PRES = {'inHg': 'in', 'mbar': 'mb', 'hPa': 'hPa'}
+UNITS_TEMP = {'degree_C': 'C', 'degree_F': 'F'}
+UNITS_RAIN = {'inch': 'in', 'mm': 'mm'}
 UNITS_WIND = {'mile_per_hour': 'mph',
               'meter_per_second': 'm/s',
-              'kilometer_per_hour': 'km/h',
               'km_per_hour': 'km/h',
               'knot': 'kts'}
-UNITS_TEMP = {'degree_C': 'C',
-              'degree_F': 'F'}
-UNITS_PRES = {'inHg': 'in',
-              'mbar': 'mb',
-              'hPa': 'hPa'}
-UNITS_RAIN = {'inch': 'in',
-              'mm': 'mm'}
-UNITS_ALT = {'foot': 'ft',
-             'meter': 'm'}
+UNITS_CLOUDBASE = {'foot': 'ft', 'meter': 'm'}
+# categorize the weewx-to-cumulus unit mappings
+UNITS = {'pressure': UNITS_PRES,
+         'temperature': UNITS_TEMP,
+         'rain': UNITS_RAIN,
+         'wind': UNITS_WIND,
+         'cloudbase': UNITS_CLOUDBASE}
+SPEED_TO_RUN = {'mile_per_hour': 'mile',
+                'meter_per_second': 'km',
+                'km_per_hour': 'km',
+                'knot': 'mile'}
 
 def _convert(from_v, from_units, to_units, group):
+    """Given a value, units and unit group convert to different units."""
     vt = (from_v, from_units, group)
     return weewx.units.convert(vt, to_units)[0]
 
+def _convert_us(from_v, from_us, to_units, obs, group):
+    """Given obs type, value, unit system and unit group convert to units."""
+    from_units = weewx.units.getStandardUnitType(from_us, obs)[0]
+    return _convert(from_v, from_units, to_units, group)
+
 def clamp_degrees(x):
+    """Convert a bearing to a Cumulus bearing.
+    
+    weewx uses 0.0 inclusive to 360.0 (exclusive) bearings whilst Cumulus 
+    uses 0.0 (exclusive) to 360.0 (inclusive) bearings. When the wind is 
+    calm Cumulus emits a wind direction of 0 degrees.
+    """
     if x is not None:
         return x if x != 0.0 else 360.0
     return None
@@ -147,8 +177,10 @@ def calc_avg_winddir_10min(dbm, ts):
                      (sts, ts))
     return clamp_degrees(val[0]) if val is not None else None
 
-# calculate wind run since midnight
 def calc_windrun(dbm, ts, db_us):
+    """Calculate windrun since midnight.  returns a value in the distance units
+    of the database (thus the temporal normalization).
+    """
     run = 0
     sod_ts = weeutil.weeutil.startOfDay(ts)
     for row in dbm.genSql("SELECT `interval`,windSpeed FROM %s "
@@ -163,8 +195,8 @@ def calc_windrun(dbm, ts, db_us):
             run += inc
     return run
 
-# get trend over past n hours, default to 3 hour window
 def get_trend(label, dbm, ts, n=3):
+    """Calculate the trend over past n hours, default to 3 hour window."""
     lastts = ts - n * 3600
     old_val = dbm.getSql("SELECT %s FROM %s "
                          "WHERE dateTime>? AND dateTime<=?" %
@@ -210,7 +242,7 @@ def calc_rain_yesterday(dbm, ts):
 def calc_rain_day(dbm, ts):
     sts = weeutil.weeutil.startOfDay(ts)
     val = dbm.getSql("SELECT SUM(rain) FROM %s "
-                     "WHERE dateTime>=? AND dateTime<=?" % dbm.table_name, 
+                     "WHERE dateTime>? AND dateTime<=?" % dbm.table_name, 
                      (sts, ts))
     return val[0] if val is not None else None
 
@@ -224,16 +256,16 @@ def calc_ET_today(dbm, ts):
 def calc_minmax(label, dbm, ts, minmax='MAX'):
     sts = weeutil.weeutil.startOfDay(ts)
     val = dbm.getSql("SELECT %s(%s) FROM %s "
-                     "WHERE dateTime>=? AND dateTime<=?" %
+                     "WHERE dateTime>? AND dateTime<=?" %
                      (minmax, label, dbm.table_name), (sts, ts))
     if val is None:
         return None, None
     t = dbm.getSql("SELECT dateTime FROM %s "
-                   "WHERE dateTime>=? AND dateTime<=? AND %s=?" %
+                   "WHERE dateTime>? AND dateTime<=? AND %s=?" %
                    (dbm.table_name, label), (sts, ts, val[0]))
     if t is None:
         return None, None
-    tstr = time.strftime("%H:%M", time.gmtime(t[0]))
+    tstr = time.strftime("%H:%M", time.localtime(t[0]))
     return val[0], tstr
 
 def calc_is_daylight(alm):
@@ -321,7 +353,8 @@ class ZambrettiForecast():
                 default_binding_dict=ZambrettiForecast.DEFAULT_BINDING_DICT)
             weewx.manager.open_manager(self.dbm_dict)
             self.forecasting_installed = True
-        except (weedb.DatabaseError, weewx.UnsupportedFeature, KeyError):
+        except (weedb.DatabaseError, weewx.UnsupportedFeature,
+                weewx.UnknownBinding, KeyError):
             pass
 
     def is_installed(self):
@@ -359,20 +392,25 @@ class CumulusRealTime(StdService):
     def __init__(self, engine, config_dict):
         super(CumulusRealTime, self).__init__(engine, config_dict)
         loginf("service version is %s" % VERSION)
-        self.altitude_ft = weewx.units.convert(engine.stn_info.altitude_vt,
-                                               "foot")[0]
+        self.altitude_m = weewx.units.convert(
+            engine.stn_info.altitude_vt, "meter")[0]
         self.latitude = engine.stn_info.latitude_f
         self.longitude = engine.stn_info.longitude_f
-        self.db_us = None  # unit system of the database
 
         d = config_dict.get('CumulusRealTime', {})
         self.filename = d.get('filename', '/var/tmp/realtime.txt')
+        loginf("output goes to %s" % self.filename)
         self.datesep = d.get('date_separator', '/')
         self.sunny_threshold = float(d.get('sunny_threshold', 0.75))
-        nonesub = d.get('none', 'NULL')
-        self.nonesub = nonesub if nonesub is not '' else 'NULL'
+        self.nonesub = d.get('none', 'NULL')
         loginf("'None' values will be displayed as %s" % self.nonesub)
-        binding = d.get('binding', 'loop').lower()
+
+        # source unit system is the database unit system
+        self.db_us = None
+        # initialise packet unit system
+        self.pkt_us = None
+
+        # get the unit system for display
         us = None
         us_label = d.get('unit_system', None)
         if us_label is not None:
@@ -383,17 +421,29 @@ class CumulusRealTime(StdService):
                 logerr("unknown unit_system %s" % us_label)
         self.unit_system = us
 
-        self.forecast = ZambrettiForecast(config_dict)
-        loginf("zambretti is installed: %s" % self.forecast.is_installed())
+        # get any overrides to the display units
+        self.units = dict()
+        for x in UNITS:
+            ukey = '%s_units' % x
+            if ukey in d:
+                if d[ukey] in UNITS[x]:
+                    loginf("%s units will be displayed as %s" % (x, d[ukey]))
+                    self.units[x] = d[ukey]
+                else:
+                    logerr("unknown unit '%s' for %s" % (d[ukey], ukey))
 
+        # configure forecasting
+        self.forecast = ZambrettiForecast(config_dict)
+        loginf("zambretti forecast: %s" % self.forecast.is_installed())
+
+        # configure the binding
+        binding = d.get('binding', 'loop').lower()
+        loginf("binding is %s" % binding)
         if binding == 'loop':
             self.bind(weewx.NEW_LOOP_PACKET, self.handle_new_loop)
         else:
             self.bind(weewx.NEW_ARCHIVE_RECORD, self.handle_new_archive)
 
-        loginf("binding is %s" % binding)
-        loginf("output goes to %s" % self.filename)
-
     def handle_new_loop(self, event):
         self.handle_data(event.packet)
 
@@ -415,105 +465,189 @@ class CumulusRealTime(StdService):
             f.write(self.create_realtime_string(data))
             f.write("\n")
 
-    def _cvt(self, from_v, to_us, obs, group):
+    # convert from database unit system to specified units
+    def _cvt(self, from_v, to_units, obs, group):
         if self.db_us is None:
             return None
-        from_units = weewx.units.getStandardUnitType(self.db_us, obs)[0]
+        return _convert_us(from_v, self.db_us, to_units, obs, group)
+
+    # convert from database unit system to specified unit system
+    def _cvt_us(self, from_v, to_us, obs, group):
         to_units = weewx.units.getStandardUnitType(to_us, obs)[0]
-        return _convert(from_v, from_units, to_units, group)
+        return self._cvt(from_v, to_units, obs, group)
+
+    # convert observation in group pressure
+    def _cvt_p(self, obs, packet, unit):
+        return _convert_us(packet.get(obs), self.pkt_us, unit, 
+                           obs, 'group_pressure')
+
+    # convert observation in group temperature
+    def _cvt_t(self, obs, packet, unit):
+        return _convert_us(packet.get(obs), self.pkt_us, unit, 
+                           obs, 'group_temperature')
+
+    # convert observation in group rainrate
+    def _cvt_rr(self, obs, packet, unit):
+        return _convert_us(packet.get(obs), self.pkt_us, unit, 
+                           obs, 'group_rainrate')
+
+    # convert observation in group speed
+    def _cvt_w(self, obs, packet, unit):
+        return _convert_us(packet.get(obs), self.pkt_us, unit, 
+                           obs, 'group_speed')
+
+    # convert observation group altitude
+    def _cvt_a(self, obs, packet, unit):
+        return _convert_us(packet.get(obs), self.pkt_us, unit, 
+                           obs, 'group_altitude')
 
     # calculate the data elements that that weewx does not provide directly.
     def calculate(self, packet, dbm):
         ts = packet.get('dateTime')
-        pkt_us = packet.get('usUnits')
-        if self.unit_system is not None and self.unit_system != pkt_us:
-            packet = weewx.units.to_std_system(packet, self.unit_system)
-            pkt_us = self.unit_system
 
         # the 'from' unit system is whatever the database is using.  get it
-        # from the database once then cache it for use in conversions.
+        # from the database once then cache it for use in conversions.  if
+        # there is not yet a database, return an empty dict and we'll get it
+        # the next time around.
         if self.db_us is None:
-            self.db_us = get_db_units(dbm)
+            try:
+                self.db_us = get_db_units(dbm)
+            except weedb.DatabaseError, e:
+                logerr("cannot determine database units: %s" % e)
+                return dict()
 
-        p_u = weewx.units.getStandardUnitType(pkt_us, 'barometer')[0]
-        t_u = weewx.units.getStandardUnitType(pkt_us, 'outTemp')[0]
-        r_u = weewx.units.getStandardUnitType(pkt_us, 'rain')[0]
-        ws_u = weewx.units.getStandardUnitType(pkt_us, 'windSpeed')[0]
-        alt_u = weewx.units.getStandardUnitType(pkt_us, 'altitude')[0]
+        # the 'to' unit system defaults to the unit system of the packet
+        # (typically the same unit system as the database, but it might not
+        # be), but if a different unit system is specified, use that...
+        self.pkt_us = packet.get('usUnits')
+        if self.unit_system is not None and self.unit_system != self.pkt_us:
+            packet = weewx.units.to_std_system(packet, self.unit_system)
+            self.pkt_us = self.unit_system
 
+        # ... then get any other specialized units.
+        p_u = self.units.get(
+            'pressure',
+            weewx.units.getStandardUnitType(self.pkt_us, 'barometer')[0])
+        t_u = self.units.get(
+            'temperature',
+            weewx.units.getStandardUnitType(self.pkt_us, 'outTemp')[0])
+        r_u = self.units.get(
+            'rain',
+            weewx.units.getStandardUnitType(self.pkt_us, 'rain')[0])
+        w_u = self.units.get(
+            'wind',
+            weewx.units.getStandardUnitType(self.pkt_us, 'windSpeed')[0])
+        cb_u = self.units.get(
+            'cloudbase',
+            weewx.units.getStandardUnitType(self.pkt_us, 'altitude')[0])
+
+        # cumulus does not use cm for rain, but weewx might
+        if r_u == 'cm':
+            r_u = 'mm'
+        # infer rain rate units from rain units
+        rr_u = '%s_per_hour' % r_u
+        # infer windrun units from wind units
+        wr_u = SPEED_TO_RUN.get(w_u, 'mile')
+
+        # now create the dictionary of data
         data = dict(packet)
-        data['windSpeed_avg'] = self._cvt(
-            calc_avg_windspeed(dbm, ts), pkt_us, 'windSpeed', 'group_speed')
-        data['cumulus_windDir'] = clamp_degrees(packet.get('windDir'))
-        data['windDir_compass'] = degree_to_compass(packet.get('windDir'))
-        v = _convert(packet.get('windSpeed'), ws_u, 'knot', 'group_speed')
-        data['windSpeed_beaufort'] = weewx.wxformulas.beaufort(v)
-        data['units_wind'] = UNITS_WIND.get(ws_u, ws_u)
+        data['units_wind'] = UNITS_WIND.get(w_u, w_u)
         data['units_temperature'] = UNITS_TEMP.get(t_u, t_u)
         data['units_pressure'] = UNITS_PRES.get(p_u, p_u)
         data['units_rain'] = UNITS_RAIN.get(r_u, r_u)
+        data['units_cloudbase'] = UNITS_CLOUDBASE.get(cb_u, cb_u)
+
+        data['barometer'] = self._cvt_p('barometer', packet, p_u)
+        data['inTemp'] = self._cvt_t('inTemp', packet, t_u)
+        data['outTemp'] = self._cvt_t('outTemp', packet, t_u)
+        data['dewpoint'] = self._cvt_t('dewpoint', packet, t_u)
+        data['heatindex'] = self._cvt_t('heatindex', packet, t_u)
+        data['humidex'] = self._cvt_t('humidex', packet, t_u)
+        data['windchill'] = self._cvt_t('windchill', packet, t_u)
+        data['appTemp'] = self._cvt_t('appTemp', packet, t_u)
+        data['windSpeed'] = self._cvt_w('windSpeed', packet, w_u)
+        data['rainRate'] = self._cvt_rr('rainRate', packet, rr_u)
+        data['cumulus_windDir'] = clamp_degrees(packet.get('windDir'))
+        data['windDir_compass'] = degree_to_compass(packet.get('windDir'))
+        data['windSpeed_avg'] = self._cvt(
+            calc_avg_windspeed(dbm, ts), w_u, 'windSpeed', 'group_speed')
+        v = _convert_us(packet.get('windSpeed'), self.pkt_us, 'knot',
+                        'windSpeed', 'group_speed')
+        data['windSpeed_beaufort'] = weewx.wxformulas.beaufort(v)
         wr = calc_windrun(dbm, ts, self.db_us)
-        data['windrun'] = self._cvt(wr, pkt_us, 'windrun', 'group_distance')
+        data['windrun'] = self._cvt(wr, wr_u, 'windrun', 'group_distance')
+        # weewx does not know of nautical miles so if wind speed units are knot
+        # then we have a windrun in miles and we need to manually convert it to
+        # nautical miles
+        if w_u == 'knot' and data['windrun'] is not None:
+            data['windrun'] /= 1.15077945
+        data['cloudbase'] = self._cvt_a('cloudbase', packet, cb_u)
         p1 = packet.get('barometer')
         p2 = get_trend('barometer', dbm, ts)
-        p2 = self._cvt(p2, pkt_us, 'barometer', 'group_pressure')
+        p2 = self._cvt_us(p2, self.pkt_us, 'barometer', 'group_pressure')
         data['pressure_trend'] = calc_trend(p1, p2)
         t1 = packet.get('outTemp')
         t2 = get_trend('outTemp', dbm, ts)
-        t2 = self._cvt(t2, pkt_us, 'outTemp', 'group_temperature')
+        t2 = self._cvt_us(t2, self.pkt_us, 'outTemp', 'group_temperature')
         data['temperature_trend'] = calc_trend(t1, t2)
+
         data['rain_month'] = self._cvt(
-            calc_rain_month(dbm, ts), pkt_us, 'rain', 'group_rain')
+            calc_rain_month(dbm, ts), r_u, 'rain', 'group_rain')
         data['rain_year'] = self._cvt(
-            calc_rain_year(dbm, ts), pkt_us, 'rain', 'group_rain')
+            calc_rain_year(dbm, ts), r_u, 'rain', 'group_rain')
         data['rain_yesterday'] = self._cvt(
-            calc_rain_yesterday(dbm, ts), pkt_us, 'rain', 'group_rain')
+            calc_rain_yesterday(dbm, ts), r_u, 'rain', 'group_rain')
         data['dayRain'] = self._cvt(
-            calc_rain_day(dbm, ts), pkt_us, 'rain', 'group_rain')
+            calc_rain_day(dbm, ts), r_u, 'rain', 'group_rain')
+
         v, t = calc_minmax('outTemp', dbm, ts, 'MAX')
         data['outTemp_max'] = self._cvt(
-            v, pkt_us, 'outTemp', 'group_temperature')
+            v, t_u, 'outTemp', 'group_temperature')
         data['outTemp_max_time'] = t
         v, t = calc_minmax('outTemp', dbm, ts, 'MIN')
         data['outTemp_min'] = self._cvt(
-            v, pkt_us, 'outTemp', 'group_temperature')
+            v, t_u, 'outTemp', 'group_temperature')
         data['outTemp_min_time'] = t
-        v, t = calc_minmax('windSpeed', dbm, ts, 'MAX')
-        data['windSpeed_max'] = self._cvt(
-            v, pkt_us, 'windSpeed', 'group_speed')
-        data['windSpeed_max_time'] = t
-        v, t = calc_minmax('windGust', dbm, ts, 'MAX')
-        data['windGust_max'] = self._cvt(
-            v, pkt_us, 'windGust', 'group_speed')
-        data['windGust_max_time'] = t
         v, t = calc_minmax('barometer', dbm, ts, 'MAX')
         data['pressure_max'] = self._cvt(
-            v, pkt_us, 'barometer', 'group_pressure')
+            v, p_u, 'barometer', 'group_pressure')
         data['pressure_max_time'] = t
         v, t = calc_minmax('barometer', dbm, ts, 'MIN')
         data['pressure_min'] = self._cvt(
-            v, pkt_us, 'barometer', 'group_pressure')
+            v, p_u, 'barometer', 'group_pressure')
         data['pressure_min_time'] = t
+        v, t = calc_minmax('windSpeed', dbm, ts, 'MAX')
+        data['windSpeed_max'] = self._cvt(
+            v, w_u, 'windSpeed', 'group_speed')
+        data['windSpeed_max_time'] = t
+        v, t = calc_minmax('windGust', dbm, ts, 'MAX')
+        data['windGust_max'] = self._cvt(
+            v, w_u, 'windGust', 'group_speed')
+        data['windGust_max_time'] = t
+
         data['10min_high_gust'] = self._cvt(
-            calc_max_gust_10min(dbm, ts), pkt_us, 'windSpeed', 'group_speed')
+            calc_max_gust_10min(dbm, ts), w_u, 'windSpeed', 'group_speed')
+        v = calc_avg_winddir_10min(dbm, ts)
+        data['10min_avg_wind_bearing'] = v
+        data['avg_wind_dir'] = degree_to_compass(v)
+
+        data['rain_hour'] = self._cvt(
+            calc_rain_hour(dbm, ts), r_u, 'rain', 'group_rain')
 
         data['ET_today'] = calc_ET_today(dbm, ts)
-        data['10min_avg_wind_bearing'] = calc_avg_winddir_10min(dbm, ts)
-        data['rain_hour'] = self._cvt(
-            calc_rain_hour(dbm, ts), pkt_us, 'rain', 'group_rain')
         data['lost_sensors_contact'] = lost_sensor_contact(packet)
-        data['avg_wind_dir'] = degree_to_compass(data['10min_avg_wind_bearing'])
-        data['units_cloudbase'] = UNITS_ALT.get(alt_u)
-        t_C = _convert(packet.get('outTemp'),
-                       t_u, 'degree_C', 'group_temperature')
-        alt_m = _convert(self.altitude_ft, 'foot', 'meter', 'group_altitude')
-        p_mbar = _convert(packet.get('barometer'), p_u, 
-                          'mbar', 'group_pressure')
-        alm = weewx.almanac.Almanac(ts, self.latitude, self.longitude, alt_m,
-                                    t_C, p_mbar)
+
+        t_C = _convert_us(packet.get('outTemp'), self.pkt_us, 'degree_C',
+                          'outTemp', 'group_temperature')
+        p_mbar = _convert_us(packet.get('barometer'), self.pkt_us, 'mbar',
+                             'barometer', 'group_pressure')
+        alm = weewx.almanac.Almanac(ts, self.latitude, self.longitude,
+                                    self.altitude_m, t_C, p_mbar)
         data['is_daylight'] = calc_is_daylight(alm)
         data['sunshine_hours'] = calc_daylight_hours(alm)
-        data['is_sunny'] = data['is_daylight']
+        data['is_sunny'] = calc_is_sunny(data.get('radiation'),
+                                         data.get('max_rad'),
+                                         self.sunny_threshold)
         data['zambretti_code'] = self.forecast.get_zambretti_code()
         return data
 
@@ -534,6 +668,8 @@ class CumulusRealTime(StdService):
     # the ## indicates calculation is not yet implemented
     def create_realtime_string(self, data):
         fields = []
+        p_dp = 2 if data['units_pressure'] == 'in' else 1
+        r_dp = 2 if data['units_rain'] == 'in' else 1
         datefmt = "%%d%s%%m%s%%y" % (self.datesep, self.datesep)
         tstr = time.strftime(datefmt, time.localtime(data['dateTime']))
         fields.append(tstr)                                           # 1
@@ -545,20 +681,20 @@ class CumulusRealTime(StdService):
         fields.append(self.format(data, 'windSpeed_avg', 1))          # 6  *
         fields.append(self.format(data, 'windSpeed', 1))              # 7
         fields.append(self.format(data, 'cumulus_windDir', 0))        # 8
-        fields.append(self.format(data, 'rainRate', 1))               # 9
-        fields.append(self.format(data, 'dayRain', 1))                # 10
-        fields.append(self.format(data, 'barometer', 1))              # 11
+        fields.append(self.format(data, 'rainRate', r_dp))            # 9
+        fields.append(self.format(data, 'dayRain', r_dp))             # 10
+        fields.append(self.format(data, 'barometer', p_dp))           # 11
         fields.append(self.format(data, 'windDir_compass'))           # 12 *
         fields.append(self.format(data, 'windSpeed_beaufort'))        # 13 *
         fields.append(self.format(data, 'units_wind'))                # 14 *
         fields.append(self.format(data, 'units_temperature'))         # 15 *
         fields.append(self.format(data, 'units_pressure'))            # 16 *
         fields.append(self.format(data, 'units_rain'))                # 17 *
-        fields.append(self.format(data, 'windrun', 1))               # 18 *
-        fields.append(self.format(data, 'pressure_trend', 1))         # 19 *
-        fields.append(self.format(data, 'rain_month', 1))             # 20 *
-        fields.append(self.format(data, 'rain_year', 1))              # 21 *
-        fields.append(self.format(data, 'rain_yesterday', 1))         # 22 *
+        fields.append(self.format(data, 'windrun', 1))                # 18 *
+        fields.append(self.format(data, 'pressure_trend', p_dp))      # 19 *
+        fields.append(self.format(data, 'rain_month', r_dp))          # 20 *
+        fields.append(self.format(data, 'rain_year', r_dp))           # 21 *
+        fields.append(self.format(data, 'rain_yesterday', r_dp))      # 22 *
         fields.append(self.format(data, 'inTemp', 1))                 # 23
         fields.append(self.format(data, 'inHumidity', 0))             # 24
         fields.append(self.format(data, 'windchill', 1))              # 25
@@ -571,9 +707,9 @@ class CumulusRealTime(StdService):
         fields.append(self.format(data, 'windSpeed_max_time'))        # 32 *
         fields.append(self.format(data, 'windGust_max', 1))           # 33 *
         fields.append(self.format(data, 'windGust_max_time'))         # 34 *
-        fields.append(self.format(data, 'pressure_max', 1))           # 35 *
+        fields.append(self.format(data, 'pressure_max', p_dp))        # 35 *
         fields.append(self.format(data, 'pressure_max_time'))         # 36 *
-        fields.append(self.format(data, 'pressure_min', 1))           # 37 *
+        fields.append(self.format(data, 'pressure_min', p_dp))        # 37 *
         fields.append(self.format(data, 'pressure_min_time'))         # 38 *
         fields.append('%s' % weewx.__version__)                       # 39
         fields.append('0')                                            # 40
@@ -581,10 +717,10 @@ class CumulusRealTime(StdService):
         fields.append(self.format(data, 'heatindex', 1))              # 42 *
         fields.append(self.format(data, 'humidex', 1))                # 43 *
         fields.append(self.format(data, 'UV', 0))                     # 44
-        fields.append(self.format(data, 'ET_today', 1))               # 45 *
+        fields.append(self.format(data, 'ET_today', r_dp))            # 45 *
         fields.append(self.format(data, 'radiation', 0))              # 46
         fields.append(self.format(data, '10min_avg_wind_bearing', 0)) # 47 *
-        fields.append(self.format(data, 'rain_hour'))                 # 48 *
+        fields.append(self.format(data, 'rain_hour', r_dp))           # 48 *
         fields.append(self.format(data, 'zambretti_code'))            # 49 *
         fields.append(self.format(data, 'is_daylight'))               # 50 *
         fields.append(self.format(data, 'lost_sensors_contact'))      # 51 *
